const express = require('express');
const app = express();
const http = require('http').createServer(app);
const cors = require('cors');
const md5 = require('md5');

const bodyParser = require('body-parser');
const consign = require('consign');
const path = require('path');
const serverConfig = require('../config/ServerConfig.js');
const jwt = require('jsonwebtoken');
// const compression = require('compression');

const watch = require('./app/model/watch');
const parametrizacoes = require('./app/model/parametrizacoes');
const logins = require('./app/model/loginModel');

app.use(bodyParser.json({ limit: '50mb' }));

app.use(cors());

consign({ cwd: 'app' });

consign({ cwd: path.resolve(__dirname, 'app'), loggin: false, verbose: false })
  .then('middleware')
  .then('logs')
  .then('model')
  .then('controller')
  .then('service')
  .then('routes')
  .into(app);

app.service.SocketService.start(http);

app.post('/add', ((req, res) => {
  res.send(`Text: ${JSON.stringify(req.body)}`);
}));

app.post('/login', ((req, res) => {
  logins.getLogin({ PASSWORD: md5(req.body.password), EMAIL: req.body.email }).then((login) => {
    if (login.rows.length === 0) {
      res.send('falha');
    } else {
      res.send({
        CDOPERADOR: login.rows[0][0],
        NMOPERADOR: login.rows[0][1],
        IDOPERACESSO: login.rows[0][2]
      });
    }
  }).catch(((error) => {
    console.log(error);
    res.send('Erro');
  }));
}));

/** Rota para buscar os dados do component
 * json da tela: data/monitoring?v=1635352485854
 * Json de configurações: /data/connect?obj=monitoring
 */
app.get('/data/*', ((req, res) => {
  /** Tipo de conexões dos monitoramentos */
  if (req.path.split('/')[2] === 'connect') {
    let conexao = [];
    let ave = [];

    /** Rota simples */
    if (req.query.obj !== undefined && req.query.obj !== 'monitoring') {
      watch.getAll({ NOME: req.query.obj })
        .then((a => {
          res.send(a.rows[0][3]);
        })).catch(((error) => {
          console.log(error);
          res.send('Component não registrado.');
        }));
    }
    /** Conexões da Home, lista todas e retorna */
    else {
      watch.getConexao({ NAVEGACAO: 'component' })
        .then((a => {
          conexao = a.rows[0][0];
          ave = JSON.parse(conexao).conexao;

          /** As conexões filhos */
          watch.getConexao({ NAVEGACAO: 'monitoring' })
            .then((a => {
              a.rows.forEach((element) => {
                const conect = JSON.parse(element[0]).conexao[1];
                ave.push(conect);
              });
              const rest = JSON.parse(conexao);
              rest.conexao = ave;
              res.send(rest);

            })).catch(((error) => {
              console.log(error);
              res.send('Component não registrado.');
            }));

        })).catch(((error) => {
          console.log(error);
          res.send('Component não registrado.');
        }));
      }
  } else if (req.path.split('/')[2] === 'parametrizacoes') {
    parametrizacoes.get({ MIN_ROW: 1, MAX_ROW: 10 })
      .then((a) => {
        const parans = [];
        a.rows.forEach((element) => {
          parans.push({
            id: element[0],
            nome: element[1],
            component: element[2],
            conexao: element[3],
            status: element[4],
            navegacao: element[5],
            ordem: element[6],
            item: element[7],
          });
        });
        res.send(parans);
      }).catch((error) => {
        console.log(error);
        res.send('Erro ao buscar os components.');
      });
  } else {
    watch.getAll({ NOME: req.path.split('/')[2] })
      .then((a) => {
        /** Se for o monitoring, busque as informações de Colunas */
        if (a.rows[0][1] === 'monitoring') {
          const componente = JSON.parse(a.rows[0][2]).children[1].children[0].columns;
          watch.getNav()
            .then((colunas) => {
              if (colunas.rows.length > 0) {
                colunas.rows.forEach((col) => {
                  componente.push(JSON.parse(col[0]));
                });
              }
              const novoComponent = JSON.parse(a.rows[0][2]);
              novoComponent.children[1].children[0].columns = componente;
              res.send(novoComponent);
            });
        } else {
          res.send(a.rows[0][2]);
        }
      }).catch((error) => {
        console.log(error);
        res.send('Component não registrado.');
      });
  }
}));

app.get('/versoes', (req, res) => {
  const novoComponent = JSON.stringify(
    {
      "data":[{
        "id": "9999",
        "cd": "9999",
        "nome": "Filial A",
        "app": "-",
        "sub_app": "-",
        "v": "-",
        "vd": ""
      },
      {
        "id": "9999.1",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "2315",
        "app": "Odhen EXP",
        "sub_app": "-",
        "v": "3.5.1",
        "vd": ""
      },
      {
        "id": "9999.2",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "2310",
        "app": "Odhen Updater",
        "sub_app": "-",
        "v": "2.8.6",
        "vd": ""
      },
      {
        "id": "9999.3.1",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "9311",
        "app": "Monitoramento",
        "sub_app": "Vendas",
        "v": "0.0.1",
        "vd": "0.0.2"
      },
      {
        "id": "9999.3.2",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "9311",
        "app": "Monitoramento",
        "sub_app": "Delivery",
        "v": "0.0.2",
        "vd": "0.0.3"
      },
      {
        "id": "9999.3.3",
        "cd": "9999",
        "nome": "Filial A",
        "PARENT_PROJECT": "9999",
        "devOpsId": "9311",
        "app": "Monitoramento",
        "sub_app": "Sat",
        "v": "0.0.2",
        "vd": ""
      }]
      ,"pagination":{"page":1,"limit":10,"total":10}}
  );
  res.send(novoComponent);
});

app.put('/data/connect/portal/*', ((req, res) => {
  watch.upElemnet({ 
    STATUS: req.body.status, 
    ORDEM: req.body.ordem, 
    COMPONENT: req.body.component, 
    ITEM: req.body.item, 
    ID: req.body.id 
  }).then((a) => {
    a.connection.commit();
    res.send('Sucesso!');
  }).catch((error) => {
    console.log(error);
    res.send(error);
  });
}));

//
let customDbname = !!process.argv[2] ? process.argv[2] : '';
let customPort = !!process.argv[3] ? process.argv[3] : '';

if (customDbname.indexOf('USR_ORG_') != -1) {
  customPort = parseInt(customDbname.slice(8, customDbname.length));
}

if (!(customDbname.indexOf('USR_ORG_') != -1) && !customPort) {
  customPort = 7777;
}

if (customDbname.indexOf('ODHEN') != -1) {
  customDbname = '';
}

http.listen(customPort, (_) => {
  console.log(`Running on ${customPort} port at ${customDbname}!`);
});

/** Instancia o front */
app.use(express.static(path.resolve(__dirname, '../frontend/dist')));
/** Faz com que o node utilize as rotas do front, mas as rotas não podem ter o mesmo path */
app.use('*', express.static(path.resolve(__dirname, '../frontend/dist')));