import { IEventParam, Http, Metadata } from '@zeedhi/core';
import { AlertService, Form, LoadingService } from '@zeedhi/common';
import Router from '../router';

export class LoginController {
  public messageValue?:string;

  login:Array<Form> = [];

  boddys:any = [];

  /** */
  public gridMounted({ component }: IEventParam<Form>) {
    this.login.push(component);
  }

  public logout() {
    localStorage.removeItem('NMOPERADOR');
    localStorage.removeItem('CDOPERADOR');
    localStorage.removeItem('IDOPERACESSO');
    document.location.reload();
  }

  public logins() {
    const form = Metadata.getInstance<Form>('login-form');
    this.loginRequestMaster({
      email: form.value.email,
      password: form.value.password,
    });
  }

  public loginJson(email:string, pass:string) {
    this.boddys = {
      disableLoader: false,
      filter: [{
        name: 'EMAIL',
        operator: '=',
        value: email, // 'teknisa@digao.com.br',
      },
      {
        name: 'PASSWORD',
        operator: '=',
        value: pass, // 'Tek@31122019',
      },
      {
        name: 'PRODUCT_ID',
        operator: '=',
        value: 1776,
      },
      {
        name: 'REQUESTER_URL',
        operator: '=',
        value: 'https://retail.teknisa.cloud/login/#/login#authentication',
      },
      {
        name: 'ATTEMPTS',
        operator: '=',
        value: 1,
      },
      {
        name: 'SHOW_FULL_OPERATOR',
        operator: '=',
        value: true,
      },
      {
        name: 'HASH',
        operator: '=',
        value: null, // 'wolrm8i1one',
      },
      {
        name: 'SESSION_CHANGE',
        operator: '=',
        value: false,
      },
      {
        name: 'KEEP_CONNECTED',
        operator: '=',
        value: 'N',
      }],
      page: 1,
      requestType: 'FilterData',
      origin: {
        containerName: 'AUTHENTICATION',
        widgetName: 'LOGIN',
        containerLabel: 'Autenticação',
        widgetLabel: 'Login',
      },
    };
  }

  public loginRequest(email:string) {
    const req = Http.post('https://retail.teknisa.cloud/backend_login/index.php/login', this.boddys);
    req.then((response) => {
      LoadingService.hide();

      /** Requisição falhou */
      if (response.data.error !== undefined) {
        const err = JSON.parse(response.data.error);
        let msg;
        if (err.CODE === 'INVALID_PASSWORD') {
          msg = 'Senha inválida';
        } else {
          msg = 'Tente novamente mais tarde.';
        }
        AlertService.show({
          name: 'buttons-alert',
          text: msg,
          color: 'error',
          right: true,
          top: true,
          buttonsSlot: [
            {
              name: 'undo-button',
              component: 'ZdButton',
              icon: true,
              iconName: 'close',
              color: 'white',
              events: {
                click: () => { AlertService.hide(); },
              },
            },
          ],
        });
      } else {
        /** Requisição e dados com sucesso! */
        const res = response.data.dataset.userData;
        if (res.LOGGED === true) {
          const ds = { DSEMAILOPER: email };
          const url = 'http://bryangomes1.zeedhi.com/workfolder/login/backend/service/index.php/getOperatorOrganization';
          const org = Http.post(url, { ds }, { headers: { 'X-Requested-With': 'XMLHttpRequest' } });

          org.then((responseOrg) => {
            const orgs = JSON.parse(responseOrg);
            const organizacao = orgs.split('_');
            localStorage.setItem('ORG', organizacao[2]);
          });

          localStorage.setItem('USER_NAME', res.USER_NAME);
          localStorage.setItem('USER_EMAIL', res.USER_EMAIL);
          localStorage.setItem('USER_ID', res.USER_ID);

          Router.push({
            path: '/monitoring',
          });
        }
      }
    });
  }

  public loginRequestMaster(data: any) {
    LoadingService.show();
    const req = Http.post('/login', data);
    req.then((response) => {
      if (response.data.CDOPERADOR !== undefined) {
        localStorage.setItem('CDOPERADOR', response.data.CDOPERADOR);
        localStorage.setItem('IDOPERACESSO', response.data.IDOPERACESSO);
        localStorage.setItem('NMOPERADOR', response.data.NMOPERADOR);
        document.location.href = document.location.origin;
      } else {
        AlertService.show({
          name: 'buttons-alert',
          text: response.data.ERROR,
          color: 'error',
          right: true,
          top: true,
          buttonsSlot: [
            {
              name: 'undo-button',
              component: 'ZdButton',
              icon: true,
              iconName: 'close',
              color: 'white',
              events: {
                click: () => { AlertService.hide(); },
              },
            },
          ],
        });
      }
      LoadingService.hide();
    }).catch(() => {
      LoadingService.hide();
    });
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
