import { Modal, ModalService } from '@zeedhi/common';

import { SocketEvent } from '@/socket/socket-event';
import {
  Config,
  IDictionary,
  IEventParam,
} from '@zeedhi/core';
// import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ZdTreeGrid } from '@zeedhi/vuetify';
import { io, Socket } from 'socket.io-client';

export class ParametersController {
  public socket?: Socket;

  public title: any;

  private fullscreenModal!: Modal;

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('parameters', true);
  }

  public destroyFullscreenModal() {
    this.fullscreenModal?.destroy();
  }

  public openFullscreenModal(row:any) {
    if (row.row.parameters !== true) {
      this.fullscreenModal.show();
    }
    this.title = 'Monitoramento de Parâmetros da ';
    this.title += row.row.name;
    this.title += ' - ';
    this.title += row.row.id;
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  /** Status do Parameters */
  public getConditionalStyle({ row }: IDictionary<any>) {
    if (row.STATUS === 'true') {
      return 'green';
    }
    return 'red';
  }

  /** Status do Parameters */
  public getConditionalMessage({ row }: IDictionary<any>) {
    if (
      (row.STATUS === '') && (row.mensagem === '')
    ) {
      return 'Parameters indisponível.';
    }
    return row.mensagem;
  }

  /**
   * Método executado quando o grid é criado
   */
  public gridMounted({ component }: IEventParam<ZdTreeGrid>) {
    // inicializa o socket
    this.socket = io(Config.endPoint);

    this.socket.off(SocketEvent.CONNECTION)
      .emit('parameters', {
        slave: false,
      });

    component.datasource.loading = true;
    this.socket.off(SocketEvent.CONNECTION)
      .on('parameters', async (branc: IDictionary) => {
        const erros = [];
        let messageSat = '';
        let messageImp = '';
        let itemParent = 0;
        let id = 1;
        console.warn(branc);

        if (branc.message.data.length > 0) {
          if (branc.message.data[0]) {
            itemParent = 1;
            erros.push({
              NAME: 'SAT',
              STATUS: 'false',
              ERROR: '',
              mensagem: '',
              ID: id,
            });
            branc.message.data[0].forEach((element: any) => {
              if (erros.length >= 2) {
                return false;
              }
              if (element.CDCODATIVASAT === null) {
                messageSat = 'Código de ativação não cadastrado.';
                erros.push({
                  STATUS: 'false',
                  ERROR: messageSat,
                  mensagem: messageSat,
                  ID: id += 1,
                  PARENT_ITEM: itemParent,
                });
              }
              if (element.DSSATHOST === null) {
                messageSat = 'Host não cadastrado.';
                erros.push({
                  STATUS: 'false',
                  ERROR: messageSat,
                  mensagem: messageSat,
                  ID: id += 1,
                  PARENT_ITEM: itemParent,
                });
              }
              return false;
            });
          }

          if (branc.message.data[1]) {
            erros.push({
              NAME: 'IMPRESSORA',
              STATUS: 'false',
              ERROR: messageImp,
              mensagem: messageImp,
              ID: id + 1,
            });
            itemParent = id + 1;
            branc.message.data[1].forEach((element: any) => {
              messageImp = '';
              if (erros.length >= 6) {
                return false;
              }
              if (element.DSIPIMPR === null) {
                messageImp = 'IP da impressora não cadastrado.';
                erros.push({
                  STATUS: 'false',
                  ERROR: messageImp,
                  mensagem: messageImp,
                  ID: id,
                  PARENT_ITEM: itemParent,
                });
              }
              if (element.DSIPPONTE === null) {
                messageImp = 'Host Periféricos não cadastrado.';
                erros.push({
                  STATUS: 'false',
                  ERROR: messageImp,
                  mensagem: messageImp,
                  ID: id,
                  PARENT_ITEM: itemParent,
                });
              }
              return false;
            });
          }
          component.datasource.loading = false;
          await component.datasource.updateData(erros);
        }
        await component.datasource.get();
      });
  }

  /**
   * Método executado logo antes do grid ser destruído
   */
  public gridBeforeDestroy() {

  }
}
